# Copyright 2010 Jacob Kaplan-Moss
# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

###
# This code is taken from python-novaclient. Goal is minimal modification.
###

"""
Command-line interface to the OpenStack Sahara API.
"""

from __future__ import print_function
import argparse
import getpass
import logging
import sys
import warnings

import six


HAS_KEYRING = False
all_errors = ValueError
try:
    import keyring
    HAS_KEYRING = True
    try:
        if isinstance(keyring.get_keyring(), keyring.backend.GnomeKeyring):
            import gnomekeyring
            all_errors = (ValueError,
                          gnomekeyring.IOError,
                          gnomekeyring.NoKeyringDaemonError)
    except Exception:
        pass
except ImportError:
    pass

from keystoneauth1.identity.generic import password
from keystoneauth1.identity.generic import token
from keystoneauth1.loading import session
from keystoneclient.auth.identity import v3 as identity
from oslo_utils import encodeutils
from oslo_utils import strutils

from saharaclient.api import client
from saharaclient.api import shell as shell_api
from saharaclient.openstack.common.apiclient import auth
from saharaclient.openstack.common.apiclient import exceptions as exc
from saharaclient.openstack.common import cliutils
from saharaclient import version

DEFAULT_API_VERSION = 'api'
DEFAULT_ENDPOINT_TYPE = 'publicURL'
DEFAULT_SERVICE_TYPE = 'data-processing'

logger = logging.getLogger(__name__)


def positive_non_zero_float(text):
    if text is None:
        return None
    try:
        value = float(text)
    except ValueError:
        msg = "%s must be a float" % text
        raise argparse.ArgumentTypeError(msg)
    if value <= 0:
        msg = "%s must be greater than 0" % text
        raise argparse.ArgumentTypeError(msg)
    return value


class SecretsHelper(object):
    def __init__(self, args, client):
        self.args = args
        self.client = client
        self.key = None

    def _validate_string(self, text):
        if text is None or len(text) == 0:
            return False
        return True

    def _make_key(self):
        if self.key is not None:
            return self.key
        keys = [
            self.client.auth_url,
            self.client.projectid,
            self.client.user,
            self.client.region_name,
            self.client.endpoint_type,
            self.client.service_type,
            self.client.service_name,
            self.client.volume_service_name,
        ]
        for (index, key) in enumerate(keys):
            if key is None:
                keys[index] = '?'
            else:
                keys[index] = str(keys[index])
        self.key = "/".join(keys)
        return self.key

    def _prompt_password(self, verify=True):
        pw = None
        if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
            # Check for Ctl-D
            try:
                while True:
                    pw1 = getpass.getpass('OS Password: ')
                    if verify:
                        pw2 = getpass.getpass('Please verify: ')
                    else:
                        pw2 = pw1
                    if pw1 == pw2 and self._validate_string(pw1):
                        pw = pw1
                        break
            except EOFError:
                pass
        return pw

    def save(self, auth_token, management_url, tenant_id):
        if not HAS_KEYRING or not self.args.os_cache:
            return
        if (auth_token == self.auth_token and
           management_url == self.management_url):
            # Nothing changed....
            return
        if not all([management_url, auth_token, tenant_id]):
            raise ValueError("Unable to save empty management url/auth token")
        value = "|".join([str(auth_token),
                          str(management_url),
                          str(tenant_id)])
        keyring.set_password("saharaclient_auth", self._make_key(), value)

    @property
    def password(self):
        if self._validate_string(self.args.os_password):
            return self.args.os_password
        verify_pass = (
            strutils.bool_from_string(cliutils.env("OS_VERIFY_PASSWORD"))
        )
        return self._prompt_password(verify_pass)

    @property
    def management_url(self):
        if not HAS_KEYRING or not self.args.os_cache:
            return None
        management_url = None
        try:
            block = keyring.get_password('saharaclient_auth',
                                         self._make_key())
            if block:
                _token, management_url, _tenant_id = block.split('|', 2)
        except all_errors:
            pass
        return management_url

    @property
    def auth_token(self):
        # Now is where it gets complicated since we
        # want to look into the keyring module, if it
        # exists and see if anything was provided in that
        # file that we can use.
        if not HAS_KEYRING or not self.args.os_cache:
            return None
        token = None
        try:
            block = keyring.get_password('saharaclient_auth',
                                         self._make_key())
            if block:
                token, _management_url, _tenant_id = block.split('|', 2)
        except all_errors:
            pass
        return token

    @property
    def tenant_id(self):
        if not HAS_KEYRING or not self.args.os_cache:
            return None
        tenant_id = None
        try:
            block = keyring.get_password('saharaclient_auth',
                                         self._make_key())
            if block:
                _token, _management_url, tenant_id = block.split('|', 2)
        except all_errors:
            pass
        return tenant_id


class SaharaClientArgumentParser(argparse.ArgumentParser):

    def __init__(self, *args, **kwargs):
        super(SaharaClientArgumentParser, self).__init__(*args, **kwargs)

    def error(self, message):
        """error(message: string)

        Prints a usage message incorporating the message to stderr and
        exits.
        """
        self.print_usage(sys.stderr)
        # FIXME(lzyeval): if changes occur in argparse.ArgParser._check_value
        choose_from = ' (choose from'
        progparts = self.prog.partition(' ')
        self.exit(2, "error: %(errmsg)s\nTry '%(mainp)s help %(subp)s'"
                     " for more information.\n" %
                     {'errmsg': message.split(choose_from)[0],
                      'mainp': progparts[0],
                      'subp': progparts[2]})


class OpenStackSaharaShell(object):

    def get_base_parser(self):
        parser = SaharaClientArgumentParser(
            prog='sahara',
            description=__doc__.strip(),
            epilog='See "sahara help COMMAND" '
                   'for help on a specific command.',
            add_help=False,
            formatter_class=OpenStackHelpFormatter,
        )

        # Global arguments
        parser.add_argument('-h', '--help',
                            action='store_true',
                            help=argparse.SUPPRESS)

        parser.add_argument('--version',
                            action='version',
                            version=version.version_info.version_string())

        parser.add_argument('--debug',
                            default=False,
                            action='store_true',
                            help="Print debugging output.")

        parser.add_argument('--os-cache',
                            default=strutils.bool_from_string(
                                cliutils.env('OS_CACHE', default=False)),
                            action='store_true',
                            help="Use the auth token cache. Defaults to False "
                            "if env[OS_CACHE] is not set.")


# TODO(mattf) - add get_timings support to Client
#        parser.add_argument('--timings',
#            default=False,
#            action='store_true',
#            help="Print call timing info")

# TODO(mattf) - use timeout
#        parser.add_argument('--timeout',
#            default=600,
#            metavar='<seconds>',
#            type=positive_non_zero_float,
#            help="Set HTTP call timeout (in seconds)")

        parser.add_argument('--region-name',
                            metavar='<region-name>',
                            default=cliutils.env('SAHARA_REGION_NAME',
                                                 'OS_REGION_NAME'),
                            help='Defaults to env[OS_REGION_NAME].')
        parser.add_argument('--region_name',
                            help=argparse.SUPPRESS)

        parser.add_argument('--service-type',
                            metavar='<service-type>',
                            help='Defaults to data-processing for all '
                                 'actions.')
        parser.add_argument('--service_type',
                            help=argparse.SUPPRESS)

# NA
#        parser.add_argument('--service-name',
#            metavar='<service-name>',
#            default=utils.env('SAHARA_SERVICE_NAME'),
#            help='Defaults to env[SAHARA_SERVICE_NAME]')
#        parser.add_argument('--service_name',
#            help=argparse.SUPPRESS)

# NA
#        parser.add_argument('--volume-service-name',
#            metavar='<volume-service-name>',
#            default=utils.env('NOVA_VOLUME_SERVICE_NAME'),
#            help='Defaults to env[NOVA_VOLUME_SERVICE_NAME]')
#        parser.add_argument('--volume_service_name',
#            help=argparse.SUPPRESS)

        parser.add_argument('--endpoint-type',
                            metavar='<endpoint-type>',
                            default=cliutils.env(
                                'SAHARA_ENDPOINT_TYPE',
                                'OS_ENDPOINT_TYPE',
                                default=DEFAULT_ENDPOINT_TYPE),
                            help=('Defaults to env[SAHARA_ENDPOINT_TYPE] or'
                                  ' env[OS_ENDPOINT_TYPE] or ')
                            + DEFAULT_ENDPOINT_TYPE + '.')
        # NOTE(dtroyer): We can't add --endpoint_type here due to argparse
        #                thinking usage-list --end is ambiguous; but it
        #                works fine with only --endpoint-type present
        #                Go figure.  I'm leaving this here for doc purposes.
        # parser.add_argument('--endpoint_type',
        #    help=argparse.SUPPRESS)

        parser.add_argument('--sahara-api-version',
                            metavar='<sahara-api-ver>',
                            default=cliutils.env(
                                'SAHARA_API_VERSION',
                                default=DEFAULT_API_VERSION),
                            help='Accepts "api", '
                                 'defaults to env[SAHARA_API_VERSION].')
        parser.add_argument('--sahara_api_version',
                            help=argparse.SUPPRESS)

        parser.add_argument('--bypass-url',
                            metavar='<bypass-url>',
                            default=cliutils.env('BYPASS_URL', default=None),
                            dest='bypass_url',
                            help="Use this API endpoint instead of the "
                            "Service Catalog.")
        parser.add_argument('--bypass_url',
                            help=argparse.SUPPRESS)

        parser.add_argument('--os-tenant-name',
                            default=cliutils.env('OS_TENANT_NAME'),
                            help='Defaults to env[OS_TENANT_NAME].')

        parser.add_argument('--os-tenant-id',
                            default=cliutils.env('OS_TENANT_ID'),
                            help='Defaults to env[OS_TENANT_ID].')

        parser.add_argument('--os-auth-system',
                            default=cliutils.env('OS_AUTH_SYSTEM'),
                            help='Defaults to env[OS_AUTH_SYSTEM].')

        parser.add_argument('--os-auth-token',
                            default=cliutils.env('OS_AUTH_TOKEN'),
                            help='Defaults to env[OS_AUTH_TOKEN].')

        # Use Keystoneclient/Keystoneauth API to parse authentication arguments
        session.Session().register_argparse_arguments(parser)
        identity.Password.register_argparse_arguments(parser)

        return parser

    def get_subcommand_parser(self, version):
        parser = self.get_base_parser()

        self.subcommands = {}
        subparsers = parser.add_subparsers(metavar='<subcommand>')

        try:
            actions_module = {
                'api': shell_api,
            }[version]
        except KeyError:
            actions_module = shell_api
        actions_module = shell_api

        self._find_actions(subparsers, actions_module)
        self._find_actions(subparsers, self)

        self._add_bash_completion_subparser(subparsers)

        return parser

    def _add_bash_completion_subparser(self, subparsers):
        subparser = (
            subparsers.add_parser('bash_completion',
                                  add_help=False,
                                  formatter_class=OpenStackHelpFormatter)
        )
        self.subcommands['bash_completion'] = subparser
        subparser.set_defaults(func=self.do_bash_completion)

    def _find_actions(self, subparsers, actions_module):
        for attr in (a for a in dir(actions_module) if a.startswith('do_')):
            # I prefer to be hyphen-separated instead of underscores.
            command = attr[3:].replace('_', '-')
            callback = getattr(actions_module, attr)
            desc = callback.__doc__ or ''
            action_help = desc.strip()
            arguments = getattr(callback, 'arguments', [])

            subparser = (
                subparsers.add_parser(command,
                                      help=action_help,
                                      description=desc,
                                      add_help=False,
                                      formatter_class=OpenStackHelpFormatter)
            )
            subparser.add_argument('-h', '--help',
                                   action='help',
                                   help=argparse.SUPPRESS,)
            self.subcommands[command] = subparser
            for (args, kwargs) in arguments:
                subparser.add_argument(*args, **kwargs)
            subparser.set_defaults(func=callback)

    def setup_debugging(self, debug):
        if not debug:
            return

        streamformat = "%(levelname)s (%(module)s:%(lineno)d) %(message)s"
        # Set up the root logger to debug so that the submodules can
        # print debug messages
        logging.basicConfig(level=logging.DEBUG,
                            format=streamformat)

    def _get_keystone_auth(self, session, auth_url, **kwargs):
        auth_token = kwargs.pop('auth_token', None)
        if auth_token:
            return token.Token(auth_url, auth_token, **kwargs)
        else:
            return password.Password(
                auth_url,
                username=kwargs.pop('username'),
                user_id=kwargs.pop('user_id'),
                password=kwargs.pop('password'),
                user_domain_id=kwargs.pop('user_domain_id'),
                user_domain_name=kwargs.pop('user_domain_name'),
                **kwargs)

    def main(self, argv):

        # Parse args once to find version and debug settings
        parser = self.get_base_parser()
        (options, args) = parser.parse_known_args(argv)
        self.setup_debugging(options.debug)
        self.options = options

        # NOTE(dtroyer): Hackery to handle --endpoint_type due to argparse
        #                thinking usage-list --end is ambiguous; but it
        #                works fine with only --endpoint-type present
        #                Go figure.
        if '--endpoint_type' in argv:
            spot = argv.index('--endpoint_type')
            argv[spot] = '--endpoint-type'

        subcommand_parser = (
            self.get_subcommand_parser(options.sahara_api_version)
        )
        self.parser = subcommand_parser

        if options.help or not argv:
            subcommand_parser.print_help()
            return 0

        args = subcommand_parser.parse_args(argv)

        # Short-circuit and deal with help right away.
        if args.func == self.do_help:
            self.do_help(args)
            return 0
        elif args.func == self.do_bash_completion:
            self.do_bash_completion(args)
            return 0

#        (os_username, os_tenant_name, os_tenant_id, os_auth_url,
#                os_region_name, os_auth_system, endpoint_type, insecure,
#                service_type, service_name, volume_service_name,
#                bypass_url, os_cache, cacert) = ( #, timeout) = (
#                        args.os_username,
#                        args.os_tenant_name, args.os_tenant_id,
#                        args.os_auth_url,
#                        args.os_region_name,
#                        args.os_auth_system,
#                        args.endpoint_type, args.insecure,
#                        args.service_type,
#                        args.service_name, args.volume_service_name,
#                        args.bypass_url, args.os_cache,
#                        args.os_cacert, args.timeout)
        (os_username, os_tenant_name, os_tenant_id,
         os_auth_url, os_auth_system, endpoint_type,
         service_type, bypass_url, os_cacert, insecure, region_name) = (
            (args.os_username, args.os_tenant_name, args.os_tenant_id,
             args.os_auth_url, args.os_auth_system, args.endpoint_type,
             args.service_type, args.bypass_url, args.os_cacert, args.insecure,
             args.region_name)
        )

        if os_auth_system and os_auth_system != "keystone":
            auth_plugin = auth.load_plugin(os_auth_system)
        else:
            auth_plugin = None

        # Fetched and set later as needed
        os_password = None

        if not endpoint_type:
            endpoint_type = DEFAULT_ENDPOINT_TYPE

        if not service_type:
            service_type = DEFAULT_SERVICE_TYPE
# NA - there is only one service this CLI accesses
#            service_type = utils.get_service_type(args.func) or service_type

        # FIXME(usrleon): Here should be restrict for project id same as
        # for os_username or os_password but for compatibility it is not.
        if not cliutils.isunauthenticated(args.func):
            if auth_plugin:
                auth_plugin.parse_opts(args)

            if not auth_plugin or not auth_plugin.opts:
                if not os_username:
                    raise exc.CommandError("You must provide a username "
                                           "via either --os-username or "
                                           "env[OS_USERNAME]")

            if not os_auth_url:
                if os_auth_system and os_auth_system != 'keystone':
                    os_auth_url = auth_plugin.get_auth_url()

            if not os_auth_url:
                    raise exc.CommandError("You must provide an auth url "
                                           "via either --os-auth-url or "
                                           "env[OS_AUTH_URL] or specify an "
                                           "auth_system which defines a "
                                           "default url with --os-auth-system "
                                           "or env[OS_AUTH_SYSTEM]")

# NA
#        if (options.os_compute_api_version and
#                options.os_compute_api_version != '1.0'):
#            if not os_tenant_name and not os_tenant_id:
#                raise exc.CommandError("You must provide a tenant name "
#                        "or tenant id via --os-tenant-name, "
#                        "--os-tenant-id, env[OS_TENANT_NAME] "
#                        "or env[OS_TENANT_ID]")
#
#            if not os_auth_url:
#                raise exc.CommandError("You must provide an auth url "
#                        "via either --os-auth-url or env[OS_AUTH_URL]")

# NOTE: The Sahara client authenticates when you create it. So instead of
#       creating here and authenticating later, which is what the novaclient
#       does, we just create the client later.

        # Now check for the password/token of which pieces of the
        # identifying keyring key can come from the underlying client
        if not cliutils.isunauthenticated(args.func):
            # NA - Client can't be used with SecretsHelper
            # helper = SecretsHelper(args, self.cs.client)
            if (auth_plugin and auth_plugin.opts and
                    "os_password" not in auth_plugin.opts):
                use_pw = False
            else:
                use_pw = True

#            tenant_id, auth_token, management_url = (helper.tenant_id,
#                                                     helper.auth_token,
#                                                     helper.management_url)
#
#            if tenant_id and auth_token and management_url:
#                self.cs.client.tenant_id = tenant_id
#                self.cs.client.auth_token = auth_token
#                self.cs.client.management_url = management_url
#                # authenticate just sets up some values in this case, no REST
#                # calls
#                self.cs.authenticate()
            if use_pw:
                # Auth using token must have failed or not happened
                # at all, so now switch to password mode and save
                # the token when its gotten... using our keyring
                # saver
                # os_password = helper.password
                os_password = args.os_password
                if not os_password:
                    raise exc.CommandError(
                        'Expecting a password provided via either '
                        '--os-password, env[OS_PASSWORD], or '
                        'prompted response')
#                self.cs.client.password = os_password
#                self.cs.client.keyring_saver = helper

        # V3 stuff
        project_info_provided = (self.options.os_tenant_name or
                                 self.options.os_tenant_id or
                                 (self.options.os_project_name and
                                  (self.options.os_project_domain_name or
                                   self.options.os_project_domain_id)) or
                                 self.options.os_project_id)

        if (not project_info_provided):
            raise exc.CommandError(
                ("You must provide a tenant_name, tenant_id, "
                 "project_id or project_name (with "
                 "project_domain_name or project_domain_id) via "
                 "  --os-tenant-name (env[OS_TENANT_NAME]),"
                 "  --os-tenant-id (env[OS_TENANT_ID]),"
                 "  --os-project-id (env[OS_PROJECT_ID])"
                 "  --os-project-name (env[OS_PROJECT_NAME]),"
                 "  --os-project-domain-id "
                 "(env[OS_PROJECT_DOMAIN_ID])"
                 "  --os-project-domain-name "
                 "(env[OS_PROJECT_DOMAIN_NAME])"))

        if not os_auth_url:
            raise exc.CommandError(
                "You must provide an auth url "
                "via either --os-auth-url or env[OS_AUTH_URL]")

        keystone_session = None
        keystone_auth = None
        if not auth_plugin:
            project_id = args.os_project_id or args.os_tenant_id
            project_name = args.os_project_name or args.os_tenant_name

            keystone_session = (session.Session().
                                load_from_argparse_arguments(args))
            keystone_auth = self._get_keystone_auth(
                keystone_session,
                args.os_auth_url,
                username=args.os_username,
                user_id=args.os_user_id,
                user_domain_id=args.os_user_domain_id,
                user_domain_name=args.os_user_domain_name,
                password=args.os_password,
                auth_token=args.os_auth_token,
                project_id=project_id,
                project_name=project_name,
                project_domain_id=args.os_project_domain_id,
                project_domain_name=args.os_project_domain_name)

        self.cs = client.Client(username=os_username,
                                api_key=os_password,
                                project_id=os_tenant_id,
                                project_name=os_tenant_name,
                                auth_url=os_auth_url,
                                sahara_url=bypass_url,
                                endpoint_type=endpoint_type,
                                session=keystone_session,
                                auth=keystone_auth,
                                cacert=os_cacert,
                                insecure=insecure,
                                service_type=service_type,
                                region_name=region_name)

        args.func(self.cs, args)

# TODO(mattf) - add get_timings support to Client
#        if args.timings:
#            self._dump_timings(self.cs.get_timings())

    def _dump_timings(self, timings):
        class Tyme(object):
            def __init__(self, url, seconds):
                self.url = url
                self.seconds = seconds
        results = [Tyme(url, end - start) for url, start, end in timings]
        total = 0.0
        for tyme in results:
            total += tyme.seconds
        results.append(Tyme("Total", total))
        cliutils.print_list(results, ["url", "seconds"], sortby_index=None)

    def do_bash_completion(self, _args):
        """Prints arguments for bash-completion.

        Prints all of the commands and options to stdout so that the
        sahara.bash_completion script doesn't have to hard code them.
        """
        commands = set()
        options = set()
        for sc_str, sc in self.subcommands.items():
            commands.add(sc_str)
            for option in sc._optionals._option_string_actions.keys():
                options.add(option)

        commands.remove('bash-completion')
        commands.remove('bash_completion')
        print(' '.join(commands | options))

    @cliutils.arg('command', metavar='<subcommand>', nargs='?',
                  help='Display help for <subcommand>.')
    def do_help(self, args):
        """Display help about this program or one of its subcommands."""
        if args.command:
            if args.command in self.subcommands:
                self.subcommands[args.command].print_help()
            else:
                raise exc.CommandError("'%s' is not a valid subcommand" %
                                       args.command)
        else:
            self.parser.print_help()


# I'm picky about my shell help.
class OpenStackHelpFormatter(argparse.HelpFormatter):
    def start_section(self, heading):
        # Title-case the headings
        heading = '%s%s' % (heading[0].upper(), heading[1:])
        super(OpenStackHelpFormatter, self).start_section(heading)


def main():
    warnings.simplefilter('once', category=DeprecationWarning)
    warnings.warn('The sahara CLI is deprecated in favor of OpenStackClient '
                  'plugin and will not be maintained anymore. '
                  'For a Python library, continue using python-saharaclient.',
                  DeprecationWarning)
    warnings.resetwarnings()
    try:
        argv = [encodeutils.safe_decode(a) for a in sys.argv[1:]]
        OpenStackSaharaShell().main(argv)

    except Exception as e:
        logger.debug(e, exc_info=1)
        print("ERROR: %s" % encodeutils.safe_encode(six.text_type(e)),
              file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import datetime
import inspect
import json
import os.path
import sys

from saharaclient.openstack.common.apiclient import exceptions
from saharaclient.openstack.common import cliutils as utils


def _print_list_field(field):
    return lambda obj: ', '.join(getattr(obj, field))


def _filter_call_args(args, func, remap={}):
    """Filter args according to func's parameter list.

    Take three arguments:
     * args - a dictionary
     * func - a function
     * remap - a dictionary
    Remove from dct all the keys which are not among the parameters
    of func. Before filtering, remap the keys in the args dict
    according to remap dict.
    """

    for name, new_name in remap.items():
        if name in args:
            args[new_name] = args[name]
            del args[name]

    valid_args = inspect.getargspec(func).args
    for name in args.keys():
        if name not in valid_args:
            print('WARNING: "%s" is not a valid parameter and will be '
                  'discarded from the request' % name)
            del args[name]


def _print_node_group_field(cluster):
    return ', '.join(map(lambda x: ': '.join(x),
                         [(node_group['name'],
                           str(node_group['count']))
                          for node_group in cluster.node_groups]))


def _show_node_group_template(template):
    template._info['node_processes'] = (
        ', '.join(template._info['node_processes'])
    )
    utils.print_dict(template._info)


def _show_cluster_template(template):
    template._info['node_groups'] = _print_node_group_field(template)
    utils.print_dict(template._info)


def _show_cluster(cluster):
    # TODO(mattf): Make this pretty, e.g format node_groups and info urls
    # Forcing wrap=47 allows for clean display on a terminal of width 80
    utils.print_dict(cluster._info, wrap=47)


def _show_job_binary_data(data):
    columns = ('id', 'name')
    utils.print_list(data, columns)


def _show_data_source(source):
    # TODO(mattf): why are we passing credentials around like this?
    if 'credentials' in source._info:
        del source._info['credentials']
    utils.print_dict(source._info)


def _show_job_binary(binary):
    # TODO(mattf): why are we passing credentials around like this?
    if 'extra' in binary._info:
        del binary._info['extra']
    utils.print_dict(binary._info)


def _show_job_template(template):
    # TODO(mattf): Make "mains" property pretty
    # TODO(mattf): handle/remove "extra" creds
    utils.print_dict(template._info)


def _show_job(job):
    # TODO(mattf): make display of info pretty, until then
    #              extract the important status information
    job._info['status'] = job._info['info']['status']
    del job._info['info']
    utils.print_dict(job._info)


def _get_by_id_or_name(manager, id=None, name=None, **kwargs):
    if not (name or id):
        raise exceptions.CommandError("either NAME or ID is required")
    if id:
        return manager.get(id, **kwargs)
    ls = manager.find(name=name)
    if len(ls) == 0:
        raise exceptions.CommandError("%s '%s' not found" %
                                      (manager.resource_class.resource_name,
                                       name))
    elif len(ls) > 1:
        raise exceptions.CommandError("%s '%s' not unique, try by ID" %
                                      (manager.resource_class.resource_name,
                                       name))
    return manager.get(ls[0].id, **kwargs)


#
# Plugins
# ~~~~~~~
# plugin-list
#
# plugin-show --name <plugin> [--version <version>]
#

def do_plugin_list(cs, args):
    """Print a list of available plugins."""
    plugins = cs.plugins.list()
    columns = ('name', 'versions', 'title')
    utils.print_list(plugins, columns,
                     {'versions': _print_list_field('versions')})


@utils.arg('--name',
           metavar='<plugin>',
           required=True,
           help='Name of the plugin.')
# TODO(mattf) - saharaclient does not support query w/ version
# @utils.arg('--version',
#           metavar='<version>',
#           help='Optional version')
def do_plugin_show(cs, args):
    """Show details of a plugin."""
    plugin = cs.plugins.get(args.name)
    plugin._info['versions'] = ', '.join(plugin._info['versions'])
    utils.print_dict(plugin._info)


#
# Image Registry
# ~~~~~~~~~~~~~~
# image-list [--tag <tag>]*
#
# image-show --name <image>|--id <image_id>
#
# image-register --name <image>|--id <image_id>
#                [--username <name>] [--description <desc>]
#
# image-unregister --name <image>|--id <image_id>
#
# image-add-tag --name <image>|--id <image_id> --tag <tag>+
#
# image-remove-tag --name <image>|--id <image_id> --tag <tag>+
#

# TODO(mattf): [--tag <tag>]*
def do_image_list(cs, args):
    """Print a list of available images."""
    images = cs.images.list()
    columns = ('name', 'id', 'username', 'tags', 'description')
    utils.print_list(images, columns, {'tags': _print_list_field('tags')})


@utils.arg('--name',
           help='Name of the image.')
@utils.arg('--id',
           metavar='<image_id>',
           help='ID of the image.')
def do_image_show(cs, args):
    """Show details of an image."""
    image = _get_by_id_or_name(cs.images, args.id, args.name)
    del image._info['metadata']
    image._info['tags'] = ', '.join(image._info['tags'])
    utils.print_dict(image._info)


# TODO(mattf): Add --name, must lookup in glance index
@utils.arg('--id',
           metavar='<image_id>',
           required=True,
           help='ID of image, run "glance image-list" to see all IDs.')
@utils.arg('--username',
           default='root',
           metavar='<name>',
           help='Username of privileged user in the image.')
@utils.arg('--description',
           default='',
           metavar='<desc>',
           help='Description of the image.')
def do_image_register(cs, args):
    """Register an image from the Image index."""
    # TODO(mattf): image register should not be through update
    cs.images.update_image(args.id, args.username, args.description)
    # TODO(mattf): No indication of result, expect image details


@utils.arg('--name',
           help='Name of the image.')
@utils.arg('--id',
           metavar='<image_id>',
           help='ID of image to unregister.')
def do_image_unregister(cs, args):
    """Unregister an image."""
    cs.images.unregister_image(
        args.id or _get_by_id_or_name(cs.images, name=args.name).id
    )
    # TODO(mattf): No indication of result, expect result to display


@utils.arg('--name',
           help='Name of the image.')
@utils.arg('--id',
           metavar='<image_id>',
           help='ID of image to tag.')
# TODO(mattf): Change --tag to --tag+
@utils.arg('--tag',
           metavar='<tag>',
           required=True,
           help='Tag to add.')
def do_image_add_tag(cs, args):
    """Add a tag to an image."""
    # TODO(mattf): Need proper add_tag API call
    id = args.id or _get_by_id_or_name(cs.images, name=args.name).id
    cs.images.update_tags(id, cs.images.get(id).tags + [args.tag, ])
    # TODO(mattf): No indication of result, expect image details


@utils.arg('--name',
           help='Name of the image.')
@utils.arg('--id',
           metavar='<image_id>',
           help='Image to tag.')
# TODO(mattf): Change --tag to --tag+
@utils.arg('--tag',
           metavar='<tag>',
           required=True,
           help='Tag to remove.')
def do_image_remove_tag(cs, args):
    """Remove a tag from an image."""
    # TODO(mattf): Need proper remove_tag API call
    id = args.id or _get_by_id_or_name(cs.images, name=args.name).id
    cs.images.update_tags(id,
                          filter(lambda x: x != args.tag,
                                 cs.images.get(id).tags))
    # TODO(mattf): No indication of result, expect image details


#
# Clusters
# ~~~~~~~~
# cluster-list
#
# cluster-show --name <cluster>|--id <cluster_id> [--json] [--show-progress]
#
# cluster-create [--json <file>]
#
# cluster-scale --name <cluster>|--id <cluster_id> [--json <file>]
#
# cluster-delete --name <cluster>|--id <cluster_id>
#

def do_cluster_list(cs, args):
    """Print a list of available clusters."""
    clusters = cs.clusters.list()
    for cluster in clusters:
        cluster.node_count = sum(map(lambda g: g['count'],
                                     cluster.node_groups))
    columns = ('name', 'id', 'status', 'node_count')
    utils.print_list(clusters, columns)


@utils.arg('--name',
           help='Name of the cluster.')
@utils.arg('--id',
           metavar='<cluster_id>',
           help='ID of the cluster to show.')
@utils.arg('--show-progress',
           help='Show provision progress events of the cluster.')
@utils.arg('--json',
           action='store_true',
           default=False,
           help='Print JSON representation of the cluster.')
def do_cluster_show(cs, args):
    """Show details of a cluster."""
    cluster = _get_by_id_or_name(cs.clusters, args.id, args.name,
                                 show_progress=args.show_progress)
    if args.json:
        print(json.dumps(cluster._info))
    else:
        _show_cluster(cluster)


@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of cluster.')
@utils.arg('--count',
           default=1,
           type=int,
           help='Number of clusters to create.')
def do_cluster_create(cs, args):
    """Create a cluster."""
    # TODO(mattf): improve template validation, e.g. template w/o name key
    template = json.loads(args.json.read())
    # The neutron_management_network parameter to clusters.create is
    # called net_id. Therefore, we must translate before invoking
    # create w/ **template. It may be desirable to simple change
    # clusters.create in the future.
    remap = {'neutron_management_network': 'net_id'}
    template['count'] = args.count
    _filter_call_args(template, cs.clusters.create, remap)

    _show_cluster(cs.clusters.create(**template))


@utils.arg('--name',
           help='Name of the cluster.')
@utils.arg('--id',
           metavar='<cluster_id>',
           help='ID of the cluster.')
@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of cluster scale.')
def do_cluster_scale(cs, args):
    """Scale a cluster."""
    cluster_id = args.id or _get_by_id_or_name(cs.clusters, name=args.name).id
    scale_template = json.loads(args.json.read())
    _show_cluster(cs.clusters.scale(cluster_id, **scale_template))


@utils.arg('--name',
           help='Name of the cluster.')
@utils.arg('--id',
           metavar='<cluster_id>',
           help='ID of the cluster to delete.')
def do_cluster_delete(cs, args):
    """Delete a cluster."""
    cs.clusters.delete(
        args.id or _get_by_id_or_name(cs.clusters, name=args.name).id
    )
    # TODO(mattf): No indication of result


#
# Node Group Templates
# ~~~~~~~~~~~~~~~~~~~~
# node-group-template-list
#
# node-group-template-show --name <template>|--id <template_id> [--json]
#
# node-group-template-create [--json <file>]
#
# node-group-template-delete --name <template>|--id <template_id>
#
# node-group-template-update --name <template>|--id <template_id> --json <file>
#

def do_node_group_template_list(cs, args):
    """Print a list of available node group templates."""
    templates = cs.node_group_templates.list()
    columns = ('name', 'id', 'plugin_name', 'node_processes', 'description')
    utils.print_list(templates, columns,
                     {'node_processes': _print_list_field('node_processes')})


@utils.arg('--name',
           help='Name of the node group template.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the node group template to show.')
@utils.arg('--json',
           action='store_true',
           default=False,
           help='Print JSON representation of node group template.')
def do_node_group_template_show(cs, args):
    """Show details of a node group template."""
    template = _get_by_id_or_name(cs.node_group_templates, args.id, args.name)
    if args.json:
        print(json.dumps(template._info))
    else:
        _show_node_group_template(template)


@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of node group template.')
def do_node_group_template_create(cs, args):
    """Create a node group template."""
    # TODO(mattf): improve template validation, e.g. template w/o name key
    template = json.loads(args.json.read())
    _filter_call_args(template, cs.node_group_templates.create)

    _show_node_group_template(cs.node_group_templates.create(**template))


@utils.arg('--name',
           help='Name of the node group template.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the node group template to delete.')
def do_node_group_template_delete(cs, args):
    """Delete a node group template."""
    cs.node_group_templates.delete(
        args.id or
        _get_by_id_or_name(cs.node_group_templates, name=args.name).id
    )
    # TODO(mattf): No indication of result


@utils.arg('--name',
           help='Name of the node group template to update.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the node group template to update.')
@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of the node group template update.')
def do_node_group_template_update(cs, args):
    """Update a node group template."""
    template = _get_by_id_or_name(cs.node_group_templates,
                                  name=args.name,
                                  id=args.id)
    update_template = json.loads(args.json.read())
    _filter_call_args(update_template, cs.node_group_templates.update)
    for param in ["plugin_name", "hadoop_version", "name", "flavor_id"]:
        if param not in update_template:
            update_template[param] = getattr(template, param, None)

    result = cs.node_group_templates.update(
        args.id or
        template._get_by_id_or_name(cs.node_group_templates,
                                    name=args.name).id,
        **update_template
    )

    _show_node_group_template(result)


#
# Cluster Templates
# ~~~~~~~~~~~~~~~~~
# cluster-template-list
#
# cluster-template-show --name <template>|--id <template_id> [--json]
#
# cluster-template-create [--json <file>]
#
# cluster-template-delete --name <template>|--id <template_id>
#

def do_cluster_template_list(cs, args):
    """Print a list of available cluster templates."""
    templates = cs.cluster_templates.list()
    columns = ('name', 'id', 'plugin_name', 'node_groups', 'description')
    utils.print_list(templates, columns,
                     {'node_groups': _print_node_group_field})


@utils.arg('--name',
           help='Name of the cluster template.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the cluster template to show.')
@utils.arg('--json',
           action='store_true',
           default=False,
           help='Print JSON representation of cluster template.')
def do_cluster_template_show(cs, args):
    """Show details of a cluster template."""
    template = _get_by_id_or_name(cs.cluster_templates, args.id, args.name)
    if args.json:
        print(json.dumps(template._info))
    else:
        _show_cluster_template(template)


@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of cluster template.')
def do_cluster_template_create(cs, args):
    """Create a cluster template."""
    # TODO(mattf): improve template validation, e.g. template w/o name key
    template = json.loads(args.json.read())
    remap = {'neutron_management_network': 'net_id'}
    _filter_call_args(template, cs.cluster_templates.create, remap)

    _show_cluster_template(cs.cluster_templates.create(**template))


@utils.arg('--name',
           help='Name of the cluster template.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the cluster template to delete.')
def do_cluster_template_delete(cs, args):
    """Delete a cluster template."""
    cs.cluster_templates.delete(
        args.id or _get_by_id_or_name(cs.cluster_templates, name=args.name).id
    )
    # TODO(mattf): No indication of result


@utils.arg('--name',
           help='Name of the cluster template to update.')
@utils.arg('--id',
           metavar='<template_id>',
           help='ID of the cluster template to update.')
@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of cluster template update.')
def do_cluster_template_update(cs, args):
    """Update a cluster template."""
    template = _get_by_id_or_name(cs.cluster_templates,
                                  name=args.name,
                                  id=args.id)
    update_template = json.loads(args.json.read())
    _filter_call_args(update_template, cs.cluster_templates.update)
    for param in ["name", "plugin_name", "hadoop_version"]:
        if param not in update_template:
            update_template[param] = getattr(template, param, None)

    result = cs.cluster_templates.update(
        args.id or
        _get_by_id_or_name(cs.node_group_templates, name=args.name).id,
        **update_template
    )

    _show_cluster_template(result)


#
# Data Sources
# ~~~~~~~~~~~~
# data-source-list
#
# data-source-show --name <name>|--id <id>
#
# data-source-create --name <name> --type <type>
#                    --url <url>
#                    [--user <user> --password <password>]
#                    [--description <desc>]
# NB: user & password if type is swift
#
# data-source-delete --name <name>|--id <id>
#

def do_data_source_list(cs, args):
    """Print a list of available data sources."""
    sources = cs.data_sources.list()
    columns = ('name', 'id', 'type', 'description')
    utils.print_list(sources, columns)


@utils.arg('--name',
           help='Name of the data source.')
@utils.arg('--id',
           help='ID of the data source.')
def do_data_source_show(cs, args):
    """Show details of a data source."""
    _show_data_source(_get_by_id_or_name(cs.data_sources, args.id, args.name))


@utils.arg('--name',
           required=True,
           help='Name of the data source.')
@utils.arg('--type',
           required=True,
           help='Type of the data source.')
@utils.arg('--url',
           required=True,
           help='URL for the data source.')
@utils.arg('--description',
           default='',
           help='Description of the data source.')
@utils.arg('--user',
           default=None,
           help='Username for accessing the data source URL.')
@utils.arg('--password',
           default=None,
           help='Password for accessing the data source URL.')
def do_data_source_create(cs, args):
    """Create a data source that provides job input or receives job output."""
    _show_data_source(cs.data_sources.create(args.name, args.description,
                                             args.type, args.url,
                                             args.user, args.password))


@utils.arg('--name',
           help='Name of the data source.')
@utils.arg('--id',
           help='ID of data source to delete.')
def do_data_source_delete(cs, args):
    """Delete a data source."""
    cs.data_sources.delete(
        args.id or _get_by_id_or_name(cs.data_sources, name=args.name).id
    )
    # TODO(mattf): No indication of result


@utils.arg('--name',
           help="Name of the data source to update.")
@utils.arg('--id',
           help="ID of the data source to update.")
@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON containing the data source fields to update.')
def do_data_source_update(cs, args):
    """Update a data source."""
    update_data = json.loads(args.json.read())
    result = cs.data_sources.update(
        args.id or _get_by_id_or_name(cs.data_sources, name=args.name).id,
        update_data
    )
    _show_data_source(result)


#
# Job Binary Internals
# ~~~~~~~~~~~~~~~~~~~~
# job-binary-data-list
#
# job-binary-data-create [--file <file>] [--name <name>]
#
# job-binary-data-delete --id <id>
#

def do_job_binary_data_list(cs, args):
    """Print a list of internally stored job binary data."""
    _show_job_binary_data(cs.job_binary_internals.list())


@utils.arg('--file',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='Data to store.')
@utils.arg('--name',
           help="Name of the job binary internal.")
def do_job_binary_data_create(cs, args):
    """Store data in the internal DB.

    Use 'swift upload' instead of this command.
    Use this command only if Swift is not available.
    """
    if args.name:
        name = args.name
    elif args.file is not sys.stdin:
        name = os.path.basename(args.file.name)
    else:
        name = datetime.datetime.now().strftime('d%Y%m%d%H%M%S')
    # Should be %F-%T except for type validation errors
    _show_job_binary_data((cs.job_binary_internals.create(
        name,
        args.file.read()),)
    )


@utils.arg('--id',
           required=True,
           help='ID of internally stored job binary data.')
def do_job_binary_data_delete(cs, args):
    """Delete an internally stored job binary data."""
    cs.job_binary_internals.delete(args.id)
    # TODO(mattf): No indication of result
    # TODO(mattf): Appears no DB constraints for removing data used by job


#
# Job Binaries
# ~~~~~~~~~~~~
# job-binary-list
#
# job-binary-show --name <name>|--id <id>
#
# job-binary-create --name <name> --url <url>
#                   [--user <user> --password <password>]
#                   [--description <desc>]
#
# job-binary-delete --name <name>|--id <id>
#

def do_job_binary_list(cs, args):
    """Print a list of job binaries."""
    binaries = cs.job_binaries.list()
    columns = ('id', 'name', 'description')
    utils.print_list(binaries, columns)


@utils.arg('--name',
           help='Name of the job binary.')
@utils.arg('--id',
           help='ID of the job binary.')
def do_job_binary_show(cs, args):
    """Show details of a job binary."""
    _show_job_binary(_get_by_id_or_name(cs.job_binaries, args.id, args.name))


@utils.arg('--name',
           required=True,
           help='Name of the job binary.')
@utils.arg('--url',
           required=True,
           help='URL for the job binary.')
@utils.arg('--description',
           default='',
           help='Description of the job binary.')
@utils.arg('--user',
           default=None,
           help='Username for accessing the job binary URL.')
@utils.arg('--password',
           default=None,
           help='Password for accessing the job binary URL.')
def do_job_binary_create(cs, args):
    """Record a job binary."""
    # TODO(mattf): make credentials consistent w/ data source
    extra = {}
    if args.user:
        extra['user'] = args.user
    if args.password:
        extra['password'] = args.password
    _show_job_binary(cs.job_binaries.create(args.name, args.url,
                                            args.description, extra))


@utils.arg('--name',
           help='Name of the job binary.')
@utils.arg('--id',
           help='ID of the job binary to delete.')
def do_job_binary_delete(cs, args):
    """Delete a job binary."""
    cs.job_binaries.delete(
        args.id or _get_by_id_or_name(cs.job_binaries, name=args.name).id
    )
    # TODO(mattf): No indication of result


@utils.arg('--name',
           help='Name of the job binary to update.')
@utils.arg('--id',
           metavar='<job_binary_id>',
           help='ID of the job binary to update.')
@utils.arg('--json',
           default=sys.stdin,
           type=argparse.FileType('r'),
           help='JSON representation of job binary update.')
def do_job_binary_update(cs, args):
    """Update a job binary."""
    update_data = json.loads(args.json.read())
    result = cs.job_binaries.update(
        args.id or
        _get_by_id_or_name(cs.job_binaries, name=args.name).id,
        update_data
    )

    _show_job_binary(result)


#
# Jobs
# ~~~~
# job-template-list
#
# job-template-show --name <name>|--id <id>
#
# job-template-create --name <name>
#                     --type <Pig|Hive|MapReduce|Java|...>
#                     [--mains <array of string>]
#                     [--libs <array of string>]
#                     [--description <desc>]
#
# job-template-delete --name <name>|--id <id>
#

def do_job_template_list(cs, args):
    """Print a list of job templates."""
    templates = cs.jobs.list()
    columns = ('id', 'name', 'description')
    utils.print_list(templates, columns)


@utils.arg('--name',
           help='Name of the job template.')
@utils.arg('--id',
           help='ID of the job template.')
def do_job_template_show(cs, args):
    """Show details of a job template."""
    _show_job_template(_get_by_id_or_name(cs.jobs, args.id, args.name))


@utils.arg('--name',
           help='Name of the job template.')
@utils.arg('--type',
           help='Type of the job template.')
@utils.arg('--main',
           action='append',
           default=[],
           help='ID for job\'s main job-binary.')
@utils.arg('--lib',
           action='append',
           default=[],
           help='ID of job\'s lib job-binary, repeatable.')
@utils.arg('--description',
           default='',
           help='Description of the job template.')
@utils.arg('--json',
           default=None,
           type=argparse.FileType('r'),
           help='JSON representation of job template.')
def do_job_template_create(cs, args):
    """Create a job template."""
    template = json.loads(args.json.read()) if args.json else {}
    _filter_call_args(template, cs.jobs.create)
    template = {
        "name": args.name or template.get("name") or None,
        "type": args.type or template.get("type") or None,
        "mains": args.main or template.get("mains") or [],
        "libs": args.lib or template.get("libs") or [],
        "description": args.description or template.get("description") or '',
        "interface": template.get("interface") or []
    }
    if not template["name"]:
        raise Exception("name is required")
    if not template["type"]:
        raise Exception("type is required")

    _show_job_template(cs.jobs.create(**template))


@utils.arg('--name',
           help='Name of the job template.')
@utils.arg('--id',
           help='ID of the job template.')
def do_job_template_delete(cs, args):
    """Delete a job template."""
    cs.jobs.delete(
        args.id or _get_by_id_or_name(cs.jobs, name=args.name).id
    )
    # TODO(mattf): No indication of result


#
# Job Executions
# ~~~~~~~~~~~~~~
# job-list
#
# job-show --id <id>
#
# job-create --job-template <id> --cluster <id>
#            [--input-data <id>] [--output-data <id>]
#            [--param <name=value>]
#            [--arg <arg>]
#            [--config <name=value>]
#
# job-delete --id <id>
#

def do_job_list(cs, args):
    """Print a list of jobs."""
    jobs = cs.job_executions.list()
    for job in jobs:
        # why is status in info.status?
        job.status = job.info['status']
    # TODO(mattf): why can cluster_id be None?
    columns = ('id', 'cluster_id', 'start_time', 'status')
    utils.print_list(jobs, columns, sortby_index=2)


@utils.arg('--id',
           required=True,
           help='ID of the job.')
def do_job_show(cs, args):
    """Show details of a job."""
    _show_job(cs.job_executions.get(args.id))


@utils.arg('--job-template',
           required=True,
           help='ID of the job template to run.')
@utils.arg('--cluster',
           required=False,
           help='ID of the cluster to run the job in.')
@utils.arg('--input-data',
           default=None,
           help='ID of the input data source.')
@utils.arg('--output-data',
           default=None,
           help='ID of the output data source.')
@utils.arg('--param',
           metavar='name=value',
           action='append',
           default=[],
           help='Parameters to add to the job, repeatable.')
@utils.arg('--arg',
           action='append',
           default=[],
           help='Arguments to add to the job, repeatable.')
@utils.arg('--config',
           metavar='name=value',
           action='append',
           default=[],
           help='Config parameters to add to the job, repeatable.')
@utils.arg('--json',
           default=None,
           type=argparse.FileType('r'),
           help='JSON representation of the job.')
def do_job_create(cs, args):
    """Create a job."""
    job = json.loads(args.json.read()) if args.json else {}
    remap = {"job_configs": "configs"}
    _filter_call_args(job, cs.job_executions.create, remap)
    _convert = lambda ls: dict(map(lambda i: i.split('=', 1), ls))
    job = {
        "cluster_id": args.cluster or job.get("cluster_id") or None,
        "input_id": args.input_data or job.get("input_id") or None,
        "output_id": args.output_data or job.get("output_id") or None,
        "interface": job.get("interface") or [],
        "configs": job.get("configs") or {}
    }
    if any((args.config, args.param, args.arg)):
        job["configs"] = {"configs": _convert(args.config),
                          "args": args.arg,
                          "params": _convert(args.param)}
    if not job["cluster_id"]:
        raise Exception("cluster is required")
    _show_job(cs.job_executions.create(args.job_template, **job))


@utils.arg('--id',
           required=True,
           help='ID of a job.')
def do_job_delete(cs, args):
    """Delete a job."""
    cs.job_executions.delete(args.id)
    # TODO(mattf): No indication of result


#
# Job Types
# ~~~~~~~~~
# job-type-list [--type] [--plugin [--plugin-version]]
#

def _print_plugin_field(job_type):

    def plugin_version_string(plugin):
        versions = ", ".join(plugin["versions"].keys())
        if versions:
            versions = "(" + versions + ")"
        return plugin["name"] + versions

    return ", ".join(map(lambda x: plugin_version_string(x), job_type.plugins))


@utils.arg('--type',
           metavar='<job_type>',
           default=None,
           help='Report only on this job type.')
@utils.arg('--plugin',
           metavar='<plugin>',
           default=None,
           help='Report only job types supported by this plugin.')
@utils.arg('--plugin-version',
           metavar='<plugin_version>',
           default=None,
           help='Report only on job types supported by this version '
           'of a specified plugin. Only valid with --plugin.')
def do_job_type_list(cs, args):
    """Show supported job types."""
    search_opts = {}
    if args.type:
        search_opts["type"] = args.type
    if args.plugin:
        search_opts["plugin"] = args.plugin
        if args.plugin_version:
            search_opts["version"] = args.plugin_version
    elif args.plugin_version:
        raise exceptions.CommandError(
            'The --plugin-version option is only valid when '
            '--plugin is specified')

    job_types = cs.job_types.list(search_opts)
    columns = ('name', 'plugin(versions)')
    utils.print_list(job_types, columns,
                     {'plugin(versions)': _print_plugin_field})



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class ClusterTemplate(base.Resource):
    resource_name = 'Cluster Template'


class ClusterTemplateManager(base.ResourceManager):
    resource_class = ClusterTemplate
    NotUpdated = base.NotUpdated()

    def create(self, name, plugin_name, hadoop_version, description=None,
               cluster_configs=None, node_groups=None, anti_affinity=None,
               net_id=None, default_image_id=None, use_autoconfig=None,
               shares=None, is_public=None, is_protected=None):
        """Create a Cluster Template."""

        data = {
            'name': name,
            'plugin_name': plugin_name,
            'hadoop_version': hadoop_version,
        }

        self._copy_if_defined(data,
                              description=description,
                              cluster_configs=cluster_configs,
                              node_groups=node_groups,
                              anti_affinity=anti_affinity,
                              neutron_management_network=net_id,
                              default_image_id=default_image_id,
                              use_autoconfig=use_autoconfig,
                              shares=shares,
                              is_public=is_public,
                              is_protected=is_protected)

        return self._create('/cluster-templates', data, 'cluster_template')

    def update(self, cluster_template_id, name=NotUpdated,
               plugin_name=NotUpdated, hadoop_version=NotUpdated,
               description=NotUpdated, cluster_configs=NotUpdated,
               node_groups=NotUpdated, anti_affinity=NotUpdated,
               net_id=NotUpdated, default_image_id=NotUpdated,
               use_autoconfig=NotUpdated, shares=NotUpdated,
               is_public=NotUpdated, is_protected=NotUpdated):
        """Update a Cluster Template."""

        data = {}
        self._copy_if_updated(data, name=name,
                              plugin_name=plugin_name,
                              hadoop_version=hadoop_version,
                              description=description,
                              cluster_configs=cluster_configs,
                              node_groups=node_groups,
                              anti_affinity=anti_affinity,
                              neutron_management_network=net_id,
                              default_image_id=default_image_id,
                              use_autoconfig=use_autoconfig,
                              shares=shares,
                              is_public=is_public,
                              is_protected=is_protected)

        return self._update('/cluster-templates/%s' % cluster_template_id,
                            data, 'cluster_template')

    def list(self, search_opts=None):
        """Get list of Cluster Templates."""
        query = base.get_query_string(search_opts)
        return self._list('/cluster-templates%s' % query, 'cluster_templates')

    def get(self, cluster_template_id):
        """Get information about a Cluster Template."""
        return self._get('/cluster-templates/%s' % cluster_template_id,
                         'cluster_template')

    def delete(self, cluster_template_id):
        """Delete a Cluster Template."""
        self._delete('/cluster-templates/%s' % cluster_template_id)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from saharaclient.osc.v1 import utils
from saharaclient.tests.unit import base


class TestUtils(base.BaseTestCase):
    def test_prepare_data(self):
        data = {'id': '123', 'name_of_res': 'name', 'description': 'descr'}

        fields = ['id', 'name_of_res', 'description']
        expected_data = {'Description': 'descr', 'Id': '123',
                         'Name of res': 'name'}
        self.assertEqual(expected_data, utils.prepare_data(data, fields))

        fields = ['id', 'name_of_res']
        expected_data = {'Id': '123', 'Name of res': 'name'}
        self.assertEqual(expected_data, utils.prepare_data(data, fields))

        fields = ['name_of_res']
        expected_data = {'Name of res': 'name'}
        self.assertEqual(expected_data, utils.prepare_data(data, fields))

    def test_get_resource(self):
        manager = mock.Mock()

        # check case when resource id is passed
        uuid = '82065b4d-2c79-420d-adc3-310de275e922'
        utils.get_resource(manager, uuid)
        manager.get.assert_called_once_with(uuid)

        # check case when resource name is passed
        utils.get_resource(manager, 'name')
        manager.find_unique.assert_called_once_with(name='name')

    def test_get_resource_id(self):
        manager = mock.Mock()

        uuid = '82065b4d-2c79-420d-adc3-310de275e922'
        manager.find_unique.return_value = mock.Mock(id=uuid)

        # check case when resource id is passed
        res = utils.get_resource_id(manager, uuid)
        self.assertEqual(uuid, res)
        manager.get.assert_not_called()
        manager.find_unique.assert_not_called()

        # check case when resource name is passed
        res = utils.get_resource_id(manager, 'name')
        manager.find_unique.assert_called_once_with(name='name')
        self.assertEqual(uuid, res)

    def test_create_dict_from_kwargs(self):
        dict1 = utils.create_dict_from_kwargs(first='1', second=2)
        self.assertEqual({'first': '1', 'second': 2}, dict1)

        dict2 = utils.create_dict_from_kwargs(first='1', second=None)
        self.assertEqual({'first': '1'}, dict2)

        dict3 = utils.create_dict_from_kwargs(first='1', second=False)
        self.assertEqual({'first': '1', 'second': False}, dict3)

    def test_prepare_column_headers(self):
        columns1 = ['first', 'second_column']
        self.assertEqual(
            ['First', 'Second column'], utils.prepare_column_headers(columns1))

        columns2 = ['First', 'Second column']
        self.assertEqual(
            ['First', 'Second column'], utils.prepare_column_headers(columns2))

        columns3 = ['first', 'second_column']
        self.assertEqual(
            ['First', 'Second'], utils.prepare_column_headers(
                columns3, remap={'second_column': 'second'}))



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import time

import saharaclient.api.base as api_base
from saharaclient.tests.integration.configs import config as cfg
import saharaclient.tests.integration.tests.base as base
import saharaclient.tests.integration.tests.utils as ut

from neutronclient.v2_0 import client as neutron_client
import novaclient.exceptions
from novaclient.v1_1 import client as nova_client

cfg = cfg.ITConfig()

common = cfg.common_config

PROP_DESCR = '_sahara_description'
PROP_USERNAME = '_sahara_username'
PROP_TAG = '_sahara_tag_'


class ClusterTest(base.ITestBase):
    def setUp(self):
        super(ClusterTest, self).setUp()

        self.cluster = None
        self.cluster_template = None
        self.image = None
        self.created_key = False
        self.node_group_templates = []

        self.nova = nova_client.Client(
            username=common.OS_USERNAME,
            api_key=common.OS_PASSWORD,
            project_id=common.OS_PROJECT_NAME,
            auth_url=common.OS_AUTH_URL)

        # Get the network ids for the managmenet network and the
        # floating ip pool if we are using neutron
        if common.NEUTRON_ENABLED:
            self.neutron = neutron_client.Client(
                username=common.OS_USERNAME,
                password=common.OS_PASSWORD,
                tenant_name=common.OS_TENANT_NAME,
                auth_url=common.OS_AUTH_URL)
            self.floating_ip_pool = self.find_network_id(
                common.FLOATING_IP_POOL)
            self.neutron_mgmt_net = self.find_network_id(
                common.INTERNAL_NEUTRON_NETWORK)
        else:
            self.neutron = None
            self.floating_ip_pool = common.FLOATING_IP_POOL
            self.neutron_mgmt_net = None

    def find_network_id(self, netname):
        try:
            net = self.neutron.list_networks(name=netname)
            net_id = net['networks'][0]['id']
            return net_id
        except IndexError:
            raise Exception(
                '\nNetwork \'%s\' not found in network list. '
                'Please make sure you specified right network name.' % netname)

    def tearDown(self):
        super(ClusterTest, self).tearDown()
        if self.created_key:
            self.nova.keypairs.delete(self.keypair)

    def init_keypair(self):
        # Figure out what keypair to use, and track whether
        # or not we created it for this test
        self.keypair = common.USER_KEYPAIR_ID
        if not self.keypair:
            self.keypair = 'key%s' % os.getpid()
        self.created_key = False
        try:
            self.nova.keypairs.get(self.keypair)
        except novaclient.exceptions.NotFound:
            self.nova.keypairs.create(self.keypair)
            self.created_key = True

    def find_image_id(self, config):

        basic_msg = '\nImage with %s "%s" was found in image list but it was '
        'possibly not registered for Sahara. Please, make sure '
        'image was correctly registered.'

        long_msg = '\nNone of parameters of image (ID, name, tag)'
        ' was specified in configuration file of '
        'integration tests. That is why there was '
        'attempt to choose image by tag '
        '"sahara_i_tests" and image with such tag '
        'was found in image list but it was possibly '
        'not registered for Sahara. Please, make '
        'sure image was correctly registered.'

        images = self.nova.images.list()

        def try_get_image_id_and_ssh_username(image, msg):
            try:
                if not config.SSH_USERNAME:
                    return image.id, image.metadata[PROP_USERNAME]
                return image.id, config.SSH_USERNAME
            except KeyError as e:
                print(msg)
                raise(e)

        # If config.IMAGE_ID is not None then find corresponding image
        # and return its ID and username. If image not found then handle error
        if config.IMAGE_ID:
            for image in images:
                if image.id == config.IMAGE_ID:
                    return try_get_image_id_and_ssh_username(
                        image,
                        basic_msg % ('ID', image.id))
            self.fail(
                '\n\nImage with ID "%s" not found in image list. '
                'Please, make sure you specified right image ID.\n' %
                config.IMAGE_ID)

        # If config.IMAGE_NAME is not None then find corresponding image
        # and return its ID and username. If image not found then handle error
        if config.IMAGE_NAME:
            for image in images:
                if image.name == config.IMAGE_NAME:
                    return try_get_image_id_and_ssh_username(
                        image,
                        basic_msg % ('name', config.IMAGE_NAME))
            self.fail(
                '\n\nImage with name "%s" not found in image list. Please, '
                'make sure you specified right image name.\n' %
                config.IMAGE_NAME)

        # If config.IMAGE_TAG is not None then find corresponding image
        # and return its ID and username. If image not found then handle error
        if config.IMAGE_TAG:
            for image in images:
                if (image.metadata.get(PROP_TAG + '%s'
                                       % config.IMAGE_TAG)) and (
                                           image.metadata.get(PROP_TAG + (
                                               '%s' % config.PLUGIN_NAME))):
                    return try_get_image_id_and_ssh_username(
                        image,
                        basic_msg % ('tag', config.IMAGE_TAG))
            self.fail(
                '\n\nImage with tag "%s" not found in list of registered '
                'images for Sahara. Please, make sure tag "%s" was added to '
                'image and image was correctly registered.\n' %
                (config.IMAGE_TAG, config.IMAGE_TAG))

        # If config.IMAGE_ID, config.IMAGE_NAME and
        # config.IMAGE_TAG are None then image is chosen
        # by tag "sahara_i_tests". If image has tag "sahara_i_tests"
        # (at the same time image ID, image name and image tag were not
        # specified in configuration file of integration tests) then return
        # its ID and username. Found image will be chosen as image for tests.
        # If image with tag "sahara_i_tests" not found then handle error
        for image in images:
            if (image.metadata.get(PROP_TAG + 'sahara_i_tests')) and (
                    image.metadata.get(PROP_TAG + (
                                       '%s' % config.PLUGIN_NAME))):
                return try_get_image_id_and_ssh_username(image, long_msg)
        self.fail(
            '\n\nNone of parameters of image (ID, name, tag) was specified in '
            'configuration file of integration tests. That is why there was '
            'attempt to choose image by tag "sahara_i_tests" but image with '
            'such tag not found in list of registered images for Sahara. '
            'Please, make sure image was correctly registered. Please, '
            'specify one of parameters of image (ID, name or tag) in '
            'configuration file of integration tests.\n'
        )

    def build_cluster(self, config, node_group_info):
        self.init_keypair()
        cluster_name = "%s-%s-%s" % (common.CLUSTER_NAME,
                                     config.PLUGIN_NAME,
                                     config.HADOOP_VERSION.replace(".", ""))
        # Create and tag an image
        image_id, username = self.find_image_id(config)
        self.cli.register_image(image_id, username, cluster_name)
        self.image = self.util.find_image_by_id(image_id)
        self.assertEqual(cluster_name, self.image.description)

        for t in (config.PLUGIN_NAME, config.HADOOP_VERSION):
            self.cli.tag_image(self.image.id, t)
            self.image = self.util.find_image_by_id(self.image.id)
            self.assertIn(t, self.image.tags)

        for ng_info in node_group_info:
            # Create node group templates
            f = self.util.generate_json_file(ng_info["values"])
            self.cli.node_group_template_create(f.name)
            t = self.util.find_node_group_template_by_name(
                ng_info["values"]["name"])
            self.assertIsNotNone(t)
            self.check_dict_elems_in_obj(ng_info["values"], t)
            self.node_group_templates.append(
                {
                    "name": t.name,
                    "node_group_template_id": t.id,
                    "count": ng_info["count"]
                }
            )

        # Create cluster template
        cluster_temp_dict = {"name": cluster_name,
                             "plugin_name": config.PLUGIN_NAME,
                             "hadoop_version": config.HADOOP_VERSION,
                             "node_groups": self.node_group_templates}

        f = self.util.generate_json_file(cluster_temp_dict)
        self.cli.cluster_template_create(f.name)
        self.cluster_template = self.util.find_cluster_template_by_name(
            cluster_name)
        self.assertIsNotNone(self.cluster_template)
        self.check_dict_elems_in_obj(cluster_temp_dict,
                                     self.cluster_template,
                                     exclude=['node_groups'])
        for idx in range(len(self.node_group_templates)):
            self.check_dict_is_subset(self.node_group_templates[idx],
                                      self.cluster_template.node_groups[idx])

        # Launch it
        cluster_dict = {"name": self.cluster_template.name,
                        "cluster_template_id": self.cluster_template.id,
                        "hadoop_version": config.HADOOP_VERSION,
                        "default_image_id": self.image.id,
                        "plugin_name": config.PLUGIN_NAME,
                        "user_keypair_id": self.keypair,
                        "neutron_management_network": self.neutron_mgmt_net}

        f = self.util.generate_json_file(cluster_dict)
        self.cli.cluster_create(f.name)
        self.cluster = self.util.find_cluster_by_name(
            self.cluster_template.name)
        self.assertIsNotNone(self.cluster)
        self.check_dict_elems_in_obj(cluster_dict, self.cluster)

    def launch_cluster_or_use_existing(self, config, ng_templates):
        # If existing cluster is set, find it and set self.cluster.
        skip_teardown = config.SKIP_CLUSTER_TEARDOWN
        if config.EXISTING_CLUSTER_ID:
            self.cluster = self.util.find_cluster_by_id(
                config.EXISTING_CLUSTER_ID)
        elif config.EXISTING_CLUSTER_NAME:
            self.cluster = self.util.find_cluster_by_name(
                config.EXISTING_CLUSTER_NAME)
        if self.cluster:
            # Always skip teardown if we used an existing cluster
            skip_teardown = True
            status = self.util.poll_cluster_state(self.cluster.id)
            self.assertEqual(ut.CLUSTER_STATUS_ACTIVE, status)
        else:
            try:
                self.build_cluster(config, ng_templates)
                status = self.util.poll_cluster_state(self.cluster.id)
                self.assertEqual(ut.CLUSTER_STATUS_ACTIVE, status)
            except Exception as e:
                if not skip_teardown:
                    self.teardown_via_client()
                raise(e)
            # A delay here seems necessary to make sure Oozie is active
            time.sleep(common.DELAY_AFTER_ACTIVE * 60)
        return skip_teardown

    def teardown_cluster(self):
        if self.cluster:
            self.cli.cluster_delete(self.cluster.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_cluster_by_id,
                              self.cluster.id)
            self.cluster = None

        if self.cluster_template:
            self.cli.cluster_template_delete(self.cluster_template.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_cluster_template_by_id,
                              self.cluster_template.id)
            self.cluster_template = None

        for ng in self.node_group_templates:
            self.cli.node_group_template_delete(ng["node_group_template_id"])
            self.assertRaises(api_base.APIException,
                              self.util.find_node_group_template_by_id,
                              ng["node_group_template_id"])
        self.node_group_templates = []

        if self.image:
            # After we unregister the image, the description should
            # be None
            self.cli.unregister_image(self.image.id)
            self.image = self.util.find_image_by_id(self.image.id)
            self.assertIsNone(self.image.description)

    def teardown_via_client(self):
        # This is a last attempt to clean up, not part of the test.
        # Try the cleanup and exit if something goes wrong.
        try:
            if self.cluster:
                self.util.client.clusters.delete(self.cluster.id)
                self.cluster = None
            if self.cluster_template:
                self.util.client.cluster_templates.delete(
                    self.cluster_template.id)
                self.cluster_template = None
            if self.worker:
                self.util.client.node_group_templates.delete(self.worker.id)
                self.worker = None
            if self.master:
                self.util.client.node_group_templates.delete(self.master.id)
                self.master = None
            if self.image:
                self.util.client.images.unregister_image(self.image.id)
                self.image = None
        except Exception:
            pass



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import saharaclient.tests.integration.tests.cluster as cluster
import saharaclient.tests.integration.tests.edp as edp


class FullTestDriver(edp.EDPTest, cluster.ClusterTest):

    def drive_full_test(self, config, ng_templates):
        # If we get an exception during cluster launch, the cluster has already
        # been cleaned up and we don't have to do anything here
        skip_teardown = self.launch_cluster_or_use_existing(config,
                                                            ng_templates)
        try:
            self.run_edp_jobs(config)
            if not skip_teardown:
                self.teardown_cluster()
        except Exception as e:
            # Oops.  Teardown via CLI is part of the test,
            # but something went wrong early.  Try tear down via the client.
            # TODO(tmckay): use excutils from openstack/common
            import traceback
            traceback.print_exc()
            if not skip_teardown:
                self.teardown_via_client()
            raise(e)



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import os
import sys

from oslo_config import cfg


def singleton(cls):
    instances = {}

    def get_instance():
        if cls not in instances:
            instances[cls] = cls()
        return instances[cls]

    return get_instance

COMMON_CONFIG_GROUP = cfg.OptGroup(name='COMMON')
COMMON_CONFIG_OPTS = [
    cfg.StrOpt('OS_USERNAME',
               default='admin',
               help='Username for OpenStack.'),
    cfg.StrOpt('OS_PASSWORD',
               default='admin',
               help='Password for OpenStack.'),
    cfg.StrOpt('OS_TENANT_NAME',
               default='admin',
               help='Tenant name for OpenStack.'),
    cfg.StrOpt('OS_AUTH_URL',
               default='http://127.0.0.1:5000/v2.0',
               help='URL for OpenStack.'),
    cfg.StrOpt('OS_PROJECT_NAME',
               default='admin',
               help='Project name for OpenStack.'),
    cfg.StrOpt('BYPASS_URL',
               help='The BYPASS_URL value to pass to the cli'),
    cfg.StrOpt('SWIFT_AUTH_VERSION',
               default=2,
               help='OpenStack auth version for Swift.'),
    cfg.IntOpt('CLUSTER_CREATION_TIMEOUT',
               default=10,
               help='Cluster creation timeout (in minutes); '
               'minimal value is 1.'),
    cfg.StrOpt('USER_KEYPAIR_ID',
               help='A keypair id to use during cluster launch. '
               'If the id is left blank, an id will be generated. '
               'If the id names an existing keypair, that keypair will '
               'be used. If the named keypair does not exist, it will be '
               'created and deleted after the test.'),
    cfg.IntOpt('DELAY_AFTER_ACTIVE',
               default=2,
               help='Length of time (in minutes) to '
               'wait after cluster is active before running jobs.'),
    cfg.StrOpt('FLOATING_IP_POOL',
               help='Pool name for floating IPs. If Sahara uses Nova '
                    'management network and auto assignment of IPs was '
                    'enabled then you should leave default value of this '
                    'parameter. If auto assignment was not enabled, then you '
                    'should specify value (floating IP pool name) of this '
                    'parameter. If Sahara uses Neutron management network, '
                    'then you should always specify value (floating IP pool '
                    'name) of this parameter.'),
    cfg.BoolOpt('NEUTRON_ENABLED',
                default=False,
                help='If Sahara uses Nova management network, then you '
                     'should leave default value of this flag. If Sahara '
                     'uses Neutron management network, then you should set '
                     'this flag to True and specify values of the following '
                     'parameters: FLOATING_IP_POOL and '
                     'INTERNAL_NEUTRON_NETWORK.'),
    cfg.StrOpt('INTERNAL_NEUTRON_NETWORK',
               default='private',
               help='Name for internal Neutron network.'),
    cfg.IntOpt('JOB_LAUNCH_TIMEOUT',
               default=10,
               help='Job launch timeout (in minutes); '
               'minimal value is 1.'),
    cfg.BoolOpt('INTERNAL_JOB_BINARIES',
                default=True,
                help='Store job binary data in the sahara '
                'internal database.  If this option is set to '
                'False, job binary data will be stored in swift.'),
    cfg.StrOpt('CLUSTER_NAME',
               default='test',
               help='Name for cluster.')
]


def general_cluster_config_opts(plugin_text, plugin_name, hadoop_ver,
                                skip_all=False):
    return [
        cfg.StrOpt('EXISTING_CLUSTER_ID',
                   help='The id of an existing active cluster '
                   'to use for the test instead of building one. '
                   'Cluster teardown will be skipped. This has priority '
                   'over EXISTING_CLUSTER_NAME'),
        cfg.StrOpt('EXISTING_CLUSTER_NAME',
                   help='The name of an existing active cluster '
                   'to use for the test instead of building one. '
                   'Cluster teardown will be skipped. This is superseded '
                   'by EXISTING_CLUSTER_ID'),
        cfg.StrOpt('IMAGE_ID',
                   help='ID for image which is used for cluster creation. '
                   'You can also specify image name or tag of image instead '
                   'of image ID. If you do not specify image related '
                   'parameters then the image for cluster creation will be '
                   'chosen by tag "sahara_i_tests".'),
        cfg.StrOpt('IMAGE_NAME',
                   help='Name for image which is used for cluster creation. '
                   'You can also specify image ID or tag of image instead of '
                   'image name. If you do not specify image related '
                   'parameters then the image for cluster creation will be '
                   'chosen by tag "sahara_i_tests".'),
        cfg.StrOpt('IMAGE_TAG',
                   help='Tag for image which is used for cluster creation. '
                   'You can also specify image ID or image name instead of '
                   'the image tag. If you do not specify image related '
                   'parameters, then the image for cluster creation will be '
                   'chosen by the tag "sahara_i_tests".'),
        cfg.StrOpt('SSH_USERNAME',
                   help='Username used to log into a cluster node via SSH.'),
        cfg.StrOpt('HADOOP_VERSION',
                   default='%s' % hadoop_ver,
                   help='Version of Hadoop'),
        cfg.StrOpt('PLUGIN_NAME',
                   default='%s' % plugin_name,
                   help='Name of plugin'),
        cfg.BoolOpt('SKIP_ALL_TESTS_FOR_PLUGIN',
                    default=skip_all,
                    help='If this flag is True, then all tests for the %s '
                    'plugin will be skipped.' % plugin_text),
        cfg.BoolOpt('SKIP_CLUSTER_TEARDOWN',
                    default=False,
                    help='Skip tearing down the cluster. If an existing '
                    'cluster is used it will never be torn down by the test.'),
        cfg.BoolOpt('SKIP_JAVA_EDP_TEST', default=False),
        cfg.BoolOpt('SKIP_MAPREDUCE_EDP_TEST', default=False),
        cfg.BoolOpt('SKIP_MAPREDUCE_STREAMING_EDP_TEST', default=False),
        cfg.BoolOpt('SKIP_PIG_EDP_TEST', default=False)
    ]


VANILLA_CONFIG_GROUP = cfg.OptGroup(name='VANILLA')
VANILLA_CONFIG_OPTS = general_cluster_config_opts("Vanilla",
                                                  "vanilla", "1.2.1")

VANILLA2_CONFIG_GROUP = cfg.OptGroup(name='VANILLA2')
VANILLA2_CONFIG_OPTS = general_cluster_config_opts("Vanilla2",
                                                   "vanilla", "2.3.0",
                                                   skip_all=True)

HDP_CONFIG_GROUP = cfg.OptGroup(name='HDP')
HDP_CONFIG_OPTS = general_cluster_config_opts("HDP",
                                              "hdp", "1.3.2",
                                              skip_all=True)


def register_config(config, config_group, config_opts):
    config.register_group(config_group)
    config.register_opts(config_opts, config_group)


@singleton
class ITConfig(object):
    def __init__(self):
        config = 'itest.conf'
        config_files = []
        config_path = '%s/saharaclient/tests/integration/configs/%s'
        if not os.path.exists(config_path % (os.getcwd(), config)):
            message = ('\n**************************************************'
                       '\nINFO: Configuration file "%s" not found  *\n'
                       '**************************************************'
                       % config)
            print(message, file=sys.stderr)
        else:
            config = os.path.join(
                config_path % (os.getcwd(), config)
            )
            config_files.append(config)

        register_config(cfg.CONF, COMMON_CONFIG_GROUP, COMMON_CONFIG_OPTS)
        register_config(cfg.CONF, VANILLA_CONFIG_GROUP, VANILLA_CONFIG_OPTS)
        register_config(cfg.CONF, VANILLA2_CONFIG_GROUP, VANILLA2_CONFIG_OPTS)
        register_config(cfg.CONF, HDP_CONFIG_GROUP, HDP_CONFIG_OPTS)

        cfg.CONF(
            [], project='Sahara_client_integration_tests',
            default_config_files=config_files
        )
        self.common_config = cfg.CONF.COMMON
        self.vanilla_config = cfg.CONF.VANILLA
        self.hdp_config = cfg.CONF.HDP
        self.vanilla2_config = cfg.CONF.VANILLA2



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from cliff import command
from cliff import lister
from cliff import show
from openstackclient.common import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc.v1 import utils

DATA_SOURCE_FIELDS = ['name', 'id', 'type', 'url', 'description', 'is_public',
                      'is_protected']
DATA_SOURCE_CHOICES = ["swift", "hdfs", "maprfs", "manila"]


class CreateDataSource(show.ShowOne):
    """Creates data source"""

    log = logging.getLogger(__name__ + ".CreateDataSource")

    def get_parser(self, prog_name):
        parser = super(CreateDataSource, self).get_parser(prog_name)

        parser.add_argument(
            'name',
            metavar="<name>",
            help="Name of the data source",
        )
        parser.add_argument(
            '--type',
            metavar="<type>",
            choices=DATA_SOURCE_CHOICES,
            help="Type of the data source (%s) "
                 "[REQUIRED]" % ', '.join(DATA_SOURCE_CHOICES),
            required=True
        )
        parser.add_argument(
            '--url',
            metavar="<url>",
            help="Url for the data source [REQUIRED]",
            required=True
        )
        parser.add_argument(
            '--username',
            metavar="<username>",
            help="Username for accessing the data source url"
        )
        parser.add_argument(
            '--password',
            metavar="<password>",
            help="Password for accessing the data source url"
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help="Description of the data source"
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the data source public',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the data source protected',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        description = parsed_args.description or ''
        data = client.data_sources.create(
            name=parsed_args.name, description=description,
            data_source_type=parsed_args.type, url=parsed_args.url,
            credential_user=parsed_args.username,
            credential_pass=parsed_args.password,
            is_public=parsed_args.public,
            is_protected=parsed_args.protected).to_dict()

        data = utils.prepare_data(data, DATA_SOURCE_FIELDS)

        return self.dict2columns(data)


class ListDataSources(lister.Lister):
    """Lists data sources"""

    log = logging.getLogger(__name__ + ".ListDataSources")

    def get_parser(self, prog_name):
        parser = super(ListDataSources, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--type',
            metavar="<type>",
            choices=DATA_SOURCE_CHOICES,
            help="List data sources of specific type "
                 "(%s)" % ', '.join(DATA_SOURCE_CHOICES)
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {'type': parsed_args.type} if parsed_args.type else {}

        data = client.data_sources.list(search_opts=search_opts)

        if parsed_args.long:
            columns = DATA_SOURCE_FIELDS
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('name', 'id', 'type')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowDataSource(show.ShowOne):
    """Display data source details"""

    log = logging.getLogger(__name__ + ".ShowDataSource")

    def get_parser(self, prog_name):
        parser = super(ShowDataSource, self).get_parser(prog_name)
        parser.add_argument(
            "data_source",
            metavar="<data-source>",
            help="Name or id of the data source to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.data_sources, parsed_args.data_source).to_dict()
        data = utils.prepare_data(data, DATA_SOURCE_FIELDS)

        return self.dict2columns(data)


class DeleteDataSource(command.Command):
    """Delete data source"""

    log = logging.getLogger(__name__ + ".DeleteDataSource")

    def get_parser(self, prog_name):
        parser = super(DeleteDataSource, self).get_parser(prog_name)
        parser.add_argument(
            "data_source",
            metavar="<data-source>",
            nargs="+",
            help="Name(s) or id(s) of the data source(s) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        for ds in parsed_args.data_source:
            data_source_id = utils.get_resource_id(
                client.data_sources, ds)
            client.data_sources.delete(data_source_id)
            sys.stdout.write(
                'Data Source "{ds}" has been removed '
                'successfully.\n'.format(ds=ds))


class UpdateDataSource(show.ShowOne):
    """Update data source"""

    log = logging.getLogger(__name__ + ".UpdateDataSource")

    def get_parser(self, prog_name):
        parser = super(UpdateDataSource, self).get_parser(prog_name)
        parser.add_argument(
            'data_source',
            metavar="<data-source>",
            help="Name or id of the data source",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="New name of the data source",
        )
        parser.add_argument(
            '--type',
            metavar="<type>",
            choices=DATA_SOURCE_CHOICES,
            help="Type of the data source "
                 "(%s)" % ', '.join(DATA_SOURCE_CHOICES)
        )
        parser.add_argument(
            '--url',
            metavar="<url>",
            help="Url for the data source"
        )
        parser.add_argument(
            '--username',
            metavar="<username>",
            help="Username for accessing the data source url"
        )
        parser.add_argument(
            '--password',
            metavar="<password>",
            help="Password for accessing the data source url"
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help="Description of the data source"
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            dest='is_public',
            help='Make the data source public (Visible from other tenants)',
        )
        public.add_argument(
            '--private',
            action='store_false',
            dest='is_public',
            help='Make the data source private (Visible only from this '
                 'tenant)',
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            dest='is_protected',
            help='Make the data source protected',
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            dest='is_protected',
            help='Make the data source unprotected',
        )
        parser.set_defaults(is_public=None, is_protected=None)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        credentials = {}
        if parsed_args.username:
            credentials['user'] = parsed_args.username
        if parsed_args.password:
            credentials['password'] = parsed_args.password
        if not credentials:
            credentials = None

        update_fields = utils.create_dict_from_kwargs(
            name=parsed_args.name,
            description=parsed_args.description,
            type=parsed_args.type, url=parsed_args.url,
            credentials=credentials,
            is_public=parsed_args.is_public,
            is_protected=parsed_args.is_protected)

        ds_id = utils.get_resource_id(
            client.data_sources, parsed_args.data_source)
        data = client.data_sources.update(ds_id, update_fields).data_source
        data = utils.prepare_data(data, DATA_SOURCE_FIELDS)

        return self.dict2columns(data)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from openstackclient.tests import utils as osc_utils

from saharaclient.api import cluster_templates as api_ct
from saharaclient.api import clusters as api_cl
from saharaclient.api import images as api_img
from saharaclient.api import node_group_templates as api_ngt
from saharaclient.osc.v1 import clusters as osc_cl
from saharaclient.tests.unit.osc.v1 import fakes

CLUSTER_INFO = {
    "description": "Cluster template for tests",

    "use_autoconfig": True,
    "is_default": False,
    "node_groups": [
        {
            "count": 2,
            "id": "ng_id",
            "name": "fakeng",
            "plugin_name": 'fake',
            "hadoop_version": '0.1',
            "node_group_template_id": 'ngt_id'
        }
    ],
    "hadoop_version": "0.1",
    "is_public": False,
    "plugin_name": "fake",
    "id": "cluster_id",
    "anti_affinity": [],
    "name": "fake",
    "is_protected": False,
    "cluster_template_id": "ct_id",
    "neutron_management_network": "net_id",
    "user_keypair_id": "test",
    "status": 'Active',
    "default_image_id": "img_id",
    'verification': {
        'status': 'GREEN',
        'id': 'ver_id',
        'cluster_id': 'cluster_id',
        'checks': [
            {
                'status': 'GREEN',
                'name': 'Some check'
            }
        ]
    }
}

CT_INFO = {
    "plugin_name": "fake",
    "hadoop_version": "0.1",
    "name": '"template',
    "id": "ct_id"
}

NGT_INFO = {
    'id': 'ngt_id',
    'name': 'fakeng'
}


class TestClusters(fakes.TestDataProcessing):
    def setUp(self):
        super(TestClusters, self).setUp()
        self.cl_mock = (
            self.app.client_manager.data_processing.clusters)
        self.ngt_mock = (
            self.app.client_manager.data_processing.node_group_templates)
        self.ct_mock = (
            self.app.client_manager.data_processing.cluster_templates)
        self.img_mock = (
            self.app.client_manager.data_processing.images)
        self.cl_mock.reset_mock()
        self.ngt_mock.reset_mock()
        self.ct_mock.reset_mock()
        self.img_mock.reset_mock()


class TestCreateCluster(TestClusters):
    # TODO(apavlov): check for creation with --json
    def setUp(self):
        super(TestCreateCluster, self).setUp()
        self.cl_mock.create.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)
        self.ct_mock.find_unique.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)
        self.img_mock.find_unique.return_value = api_img.Image(
            None, {'id': 'img_id'})
        self.net_mock = self.app.client_manager.network
        self.net_mock.find_network.return_value = mock.Mock(id='net_id')
        self.net_mock.reset_mock()

        # Command to test
        self.cmd = osc_cl.CreateCluster(self.app, None)

    def test_cluster_create_minimum_options(self):
        arglist = ['--name', 'fake', '--cluster-template', 'template',
                   '--image', 'ubuntu']
        verifylist = [('name', 'fake'), ('cluster_template', 'template'),
                      ('image', 'ubuntu')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.create.assert_called_once_with(
            cluster_template_id='ct_id', count=None, default_image_id='img_id',
            description=None, hadoop_version='0.1', is_protected=False,
            is_public=False, is_transient=False, name='fake', net_id=None,
            plugin_name='fake', user_keypair_id=None)

    def test_cluster_create_all_options(self):
        arglist = ['--name', 'fake', '--cluster-template', 'template',
                   '--image', 'ubuntu', '--user-keypair', 'test',
                   '--neutron-network', 'net', '--description', 'descr',
                   '--transient', '--public', '--protected']

        verifylist = [('name', 'fake'), ('cluster_template', 'template'),
                      ('image', 'ubuntu'), ('user_keypair', 'test'),
                      ('neutron_network', 'net'), ('description', 'descr'),
                      ('transient', True), ('public', True),
                      ('protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.create.assert_called_once_with(
            cluster_template_id='ct_id', count=None, default_image_id='img_id',
            description='descr', hadoop_version='0.1', is_protected=True,
            is_public=True, is_transient=True, name='fake', net_id='net_id',
            plugin_name='fake', user_keypair_id='test')

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Cluster template id',
                            'Description', 'Id', 'Image',
                            'Is protected', 'Is public', 'Name',
                            'Neutron management network', 'Node groups',
                            'Plugin name', 'Status', 'Use autoconfig',
                            'User keypair id', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'ct_id', 'Cluster template for tests',
                         'cluster_id', 'img_id', False, False, 'fake',
                         'net_id', 'fakeng:2', 'fake', 'Active', True, 'test',
                         '0.1')
        self.assertEqual(expected_data, data)

    def test_cluster_create_with_count(self):
        clusters_mock = mock.Mock()
        clusters_mock.to_dict.return_value = {
            'clusters': ['cluster1_id', 'cluster2_id']
        }
        self.cl_mock.create.return_value = clusters_mock

        arglist = ['--name', 'fake', '--cluster-template', 'template',
                   '--image', 'ubuntu', '--count', '2']
        verifylist = [('name', 'fake'), ('cluster_template', 'template'),
                      ('image', 'ubuntu'), ('count', 2)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.create.assert_called_once_with(
            cluster_template_id='ct_id', count=2, default_image_id='img_id',
            description=None, hadoop_version='0.1', is_protected=False,
            is_public=False, is_transient=False, name='fake', net_id=None,
            plugin_name='fake', user_keypair_id=None)

        # Check that columns are correct
        expected_columns = ('fake',)
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('cluster_id',)
        self.assertEqual(expected_data, data)


class TestListClusters(TestClusters):
    def setUp(self):
        super(TestListClusters, self).setUp()
        self.cl_mock.list.return_value = [api_cl.Cluster(
            None, CLUSTER_INFO)]

        # Command to test
        self.cmd = osc_cl.ListClusters(self.app, None)

    def test_clusters_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version', 'Status']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('fake', 'cluster_id', 'fake', '0.1', 'Active')]
        self.assertEqual(expected_data, list(data))

    def test_clusters_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version', 'Status',
                            'Description', 'Image']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('fake', 'cluster_id', 'fake', '0.1', 'Active',
                          'Cluster template for tests', 'img_id')]
        self.assertEqual(expected_data, list(data))

    def test_clusters_list_extra_search_opts(self):
        arglist = ['--plugin', 'fake', '--version', '0.1', '--name', 'fake']
        verifylist = [('plugin', 'fake'), ('version', '0.1'),
                      ('name', 'fake')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version', 'Status']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('fake', 'cluster_id', 'fake', '0.1', 'Active')]
        self.assertEqual(expected_data, list(data))


class TestShowCluster(TestClusters):
    def setUp(self):
        super(TestShowCluster, self).setUp()
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)

        # Command to test
        self.cmd = osc_cl.ShowCluster(self.app, None)

    def test_cluster_show(self):
        arglist = ['fake']
        verifylist = [('cluster', 'fake')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.find_unique.assert_called_once_with(name='fake')

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Cluster template id',
                            'Description', 'Id', 'Image',
                            'Is protected', 'Is public', 'Name',
                            'Neutron management network', 'Node groups',
                            'Plugin name', 'Status', 'Use autoconfig',
                            'User keypair id', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'ct_id', 'Cluster template for tests',
                         'cluster_id', 'img_id', False, False, 'fake',
                         'net_id', 'fakeng:2', 'fake', 'Active', True, 'test',
                         '0.1')
        self.assertEqual(expected_data, data)

    def test_cluster_show_verification(self):
        arglist = ['fake', '--verification']
        verifylist = [('cluster', 'fake'), ('verification', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.find_unique.assert_called_once_with(name='fake')

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Cluster template id',
                            'Description', 'Health check (some check)', 'Id',
                            'Image', 'Is protected', 'Is public', 'Name',
                            'Neutron management network', 'Node groups',
                            'Plugin name', 'Status', 'Use autoconfig',
                            'User keypair id', 'Verification status',
                            'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'ct_id', 'Cluster template for tests', 'GREEN',
                         'cluster_id', 'img_id', False, False, 'fake',
                         'net_id', 'fakeng:2', 'fake', 'Active', True, 'test',
                         'GREEN', '0.1')
        self.assertEqual(expected_data, data)


class TestDeleteCluster(TestClusters):
    def setUp(self):
        super(TestDeleteCluster, self).setUp()
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)

        # Command to test
        self.cmd = osc_cl.DeleteCluster(self.app, None)

    def test_cluster_delete(self):
        arglist = ['fake']
        verifylist = [('cluster', ['fake'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.delete.assert_called_once_with('cluster_id')


class TestUpdateCluster(TestClusters):
    def setUp(self):
        super(TestUpdateCluster, self).setUp()
        self.cl_mock.update.return_value = mock.Mock(
            cluster=CLUSTER_INFO.copy())
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)

        # Command to test
        self.cmd = osc_cl.UpdateCluster(self.app, None)

    def test_cluster_update_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_cluster_update_nothing_updated(self):
        arglist = ['fake']

        verifylist = [('cluster', 'fake')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.update.assert_called_once_with('cluster_id')

    def test_cluster_update_all_options(self):
        arglist = ['fake', '--name', 'fake', '--description', 'descr',
                   '--public', '--protected']

        verifylist = [('cluster', 'fake'), ('name', 'fake'),
                      ('description', 'descr'), ('is_public', True),
                      ('is_protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.update.assert_called_once_with(
            'cluster_id', description='descr', is_protected=True,
            is_public=True, name='fake')

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Cluster template id',
                            'Description', 'Id', 'Image',
                            'Is protected', 'Is public', 'Name',
                            'Neutron management network', 'Node groups',
                            'Plugin name', 'Status', 'Use autoconfig',
                            'User keypair id', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'ct_id', 'Cluster template for tests',
                         'cluster_id', 'img_id', False, False, 'fake',
                         'net_id', 'fakeng:2', 'fake', 'Active', True, 'test',
                         '0.1')
        self.assertEqual(expected_data, data)

    def test_cluster_update_private_unprotected(self):
        arglist = ['fake', '--private', '--unprotected']

        verifylist = [('cluster', 'fake'), ('is_public', False),
                      ('is_protected', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.update.assert_called_once_with(
            'cluster_id', is_protected=False, is_public=False)


class TestScaleCluster(TestClusters):
    def setUp(self):
        super(TestScaleCluster, self).setUp()
        self.cl_mock.scale.return_value = mock.Mock(
            cluster=CLUSTER_INFO.copy())
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)

        # Command to test
        self.cmd = osc_cl.ScaleCluster(self.app, None)

    def test_cluster_scale_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_cluster_scale_resize(self):
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)
        arglist = ['fake', '--instances', 'fakeng:1']

        verifylist = [('cluster', 'fake'),
                      ('instances', ['fakeng:1'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.scale.assert_called_once_with(
            'cluster_id',
            {'resize_node_groups': [
                {'count': 1,
                 'name': 'fakeng'}]}
        )

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Cluster template id',
                            'Description', 'Id', 'Image',
                            'Is protected', 'Is public', 'Name',
                            'Neutron management network', 'Node groups',
                            'Plugin name', 'Status', 'Use autoconfig',
                            'User keypair id', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'ct_id', 'Cluster template for tests',
                         'cluster_id', 'img_id', False, False, 'fake',
                         'net_id', 'fakeng:2', 'fake', 'Active', True, 'test',
                         '0.1')
        self.assertEqual(expected_data, data)

    def test_cluster_scale_add_ng(self):
        new_ng = {'name': 'new', 'id': 'new_id'}
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, new_ng)
        arglist = ['fake', '--instances', 'new:1']

        verifylist = [('cluster', 'fake'), ('instances', ['new:1'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.scale.assert_called_once_with(
            'cluster_id',
            {'add_node_groups': [
                {'count': 1,
                 'node_group_template_id': 'new_id',
                 'name': 'new'}
            ]})


class TestVerificationUpdateCluster(TestClusters):
    def setUp(self):
        super(TestVerificationUpdateCluster, self).setUp()
        self.cl_mock.find_unique.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)
        self.cl_mock.verification_update.return_value = api_cl.Cluster(
            None, CLUSTER_INFO)

        # Command to test
        self.cmd = osc_cl.VerificationUpdateCluster(self.app, None)

    def test_verification_show(self):
        arglist = ['fake', '--show']
        verifylist = [('cluster', 'fake'), ('show', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.find_unique.assert_called_once_with(name='fake')

        # Check that columns are correct
        expected_columns = ('Health check (some check)', 'Verification status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('GREEN', 'GREEN')
        self.assertEqual(expected_data, data)

    def test_verification_start(self):
        arglist = ['fake', '--start']
        verifylist = [('cluster', 'fake'), ('status', 'START')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.cl_mock.verification_update.assert_called_once_with(
            'cluster_id', 'START')



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six
import testtools

from saharaclient.api import base
from saharaclient.api import client

from requests_mock.contrib import fixture


class BaseTestCase(testtools.TestCase):

    URL = 'http://localhost:8386'
    TOKEN = 'token'

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.responses = self.useFixture(fixture.Fixture())
        self.client = client.Client(sahara_url=self.URL,
                                    input_auth_token=self.TOKEN)

    def assertFields(self, body, obj):
        for key, value in six.iteritems(body):
            self.assertEqual(value, getattr(obj, key))


class TestResource(base.Resource):
    resource_name = 'Test Resource'
    defaults = {'description': 'Test Description',
                'extra': "extra"}


class TestManager(base.ResourceManager):
    resource_class = TestResource
